@@ -12,6 +12,9 @@
 	   #:init-common-lisp-controller-v4
 	   #:clc-require
 	   #:clc-build-all-packages
+	   #:*clc-quiet*
+	   #:calculate-fasl-root
+	   #:list-systems
 	   ;; depricated:
 	   #:make-clc-send-command-string
 	   #:send-clc-command)
@@ -22,6 +25,8 @@
 
 (in-package #:common-lisp-controller)
 
+(defvar *clc-quiet* nil
+  "If true then clc prints no messages")
 
 ;; Some general utilities to make the
 ;; descriptions shorter
@@ -40,19 +45,8 @@
 (defvar *implementation-name* nil "The name of the implementation,
 used to name the directory in /var/cache/common-lisp-controller")
 
-(defun init-common-lisp-controller (fasl-root
-                                    &key
-                                    (source-root "/usr/share/common-lisp/")
-                                    (version 2))
-  (declare (ignore source-root version))
-  ;; vodoo: extract the name of the implementation
-  ;; from the old fasl directory... 
-  (init-common-lisp-controller-v4
-   (first
-    (last
-     (pathname-directory
-      (parse-namestring
-       fasl-root))))))
+(define-modify-macro appendf (&rest lists) append)
+
 
 (defun init-common-lisp-controller-v4 (implementation-name)
   "configures common-lisp-controller. IMPLEMENTATION-NAME
@@ -61,6 +55,9 @@ Fasl's will be created in /var/cache/common-lisp-controller/<userid>/<implementa
 
   (setf *implementation-name* implementation-name)
 
+  (pushnew :common-lisp-controller *features*)
+  (pushnew :clc-os-debian *features*)
+
   ;; force both parameters to directories...
   (let* ((fasl-root (merge-pathnames
 		     (make-pathname
@@ -91,7 +88,9 @@ Fasl's will be created in /var/cache/common-lisp-controller/<userid>/<implementa
 			     :print nil
 			     :verbose nil)
 	       ;; then load it:
-	       (load compiled-file-pathname))))
+	       (load compiled-file-pathname
+		     :verbose nil
+		     :print nil))))
       ;; first ourselves:
       (compile-and-load  "common-lisp-controller"
 			 "common-lisp-controller.lisp")
@@ -109,19 +108,32 @@ Fasl's will be created in /var/cache/common-lisp-controller/<userid>/<implementa
       (setq cl:*features* (delete :sbcl-hooks-require  cl:*features*))
 
       ;; register the systems root:
-      (pushnew *systems-root*
-	    	(symbol-value (intern (symbol-name :*central-registry*)
-					  (find-package :asdf)))
-		:test #'equal)
+      (appendf 
+           (symbol-value (intern (symbol-name :*central-registry*)
+                                 (find-package :asdf)))
+           (list *systems-root*))
 
-      (pushnew '(merge-pathnames ".clc/systems/" 
-			      	(user-homedir-pathname))
-	     	(symbol-value (intern (symbol-name :*central-registry*)
-				           (find-package :asdf)))
-		:test #'equal)))
+      ;; put the users asdf files at the FRONT
+      (pushnew '(merge-pathnames ".clc/systems/"
+			         (user-homedir-pathname))
+            (symbol-value (intern (symbol-name :*central-registry*)
+                                  (find-package :asdf)))
+	    :test #'equalp)))
   (values))
 
 
 
-
+(defun init-common-lisp-controller (fasl-root
+                                    &key
+                                    (source-root "/usr/share/common-lisp/")
+                                    (version 2))
+  (declare (ignore source-root version))
+  ;; vodoo: extract the name of the implementation
+  ;; from the old fasl directory... 
+  (init-common-lisp-controller-v4
+   (first
+    (last
+     (pathname-directory
+      (parse-namestring
+       fasl-root))))))
 
